<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';
require_once 'lib/dune_exception.php';

require_once 'dunestore_model.php';

class DuneStoreCategoryListScreen extends AbstractPreloadedRegularScreen
{
    const ID = 'category_list';

    public static function get_media_url_str($name)
    {
        return MediaURL::encode(
            array('screen_id' => self::ID, 'name' => $name));
    }

    ////////////////////////////////////////////////////////////

    private $model;

    public function __construct(DuneStoreModel $model)
    {
        parent::__construct(self::ID);
        $this->model = $model;
    }

    protected function do_get_folder_views(&$plugin_cookies)
    {
        return array(
            // Index 0.
            array
            (
                PluginRegularFolderView::async_icon_loading => false,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 4,
                    ViewParams::num_rows => 2,

                    ViewParams::icon_selection_box_dx => -123,
                    ViewParams::icon_selection_box_dy => -131,
                    ViewParams::icon_selection_box_width => 245,
                    ViewParams::icon_selection_box_height => 245,

                    ViewParams::paint_icon_badge_box => 1,

                    ViewParams::icon_badge_box_dx => -100,
                    ViewParams::icon_badge_box_dy => -110,
                    ViewParams::icon_badge_box_width => 200,
                    ViewParams::icon_badge_box_height => 200,

                    ViewParams::icon_badge_box_sel_dx => -120,
                    ViewParams::icon_badge_box_sel_dy => -121,
                    ViewParams::icon_badge_box_sel_width => 240,
                    ViewParams::icon_badge_box_sel_height => 240,

                    ViewParams::paint_details => false,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_padding_top => 6,
                    ViewItemParams::item_padding_bottom => 28,
                    ViewItemParams::item_caption_dy => -10,
                    ViewItemParams::item_caption_sel_dy => 12,

                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                    ViewItemParams::icon_fit_scale_factor => 1.0,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    array 
                    (
                        ViewItemParams::icon_path => "missing://",
                        ViewItemParams::item_badge_icon_path => "gui_skin://large_icons/2.png",
                    ),
            ),
        );
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array
        (
            GUI_EVENT_KEY_ENTER => ActionFactory::open_folder(),
        );
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        if (!isset($media_url->screen_id) ||
            $media_url->screen_id === self::ID)
        {
            $this->model->unset_model();
        }

        $this->model->ensure_loaded();

        $view_index = $this->get_folder_view_index($plugin_cookies);
        $is_list_view = $view_index == 5 || $view_index == 6;
        $use_description_in_details = $view_index == 0 || $view_index == 2 || $view_index == 5;

        $items = array();

        $ndx = 1;
        foreach ($this->model->get_available_categories() as $category)
        {
#            hd_print('name = ' . $category->name);
#            hd_print('caption = ' . $category->caption);
#            hd_print('icon_url = ' . $category->icon_url);

            $item_detailed_info = $category->description;

            $icon_path = $category->icon_url;

            if ($category->type == 'categories')
            {
                $item_media_url = self::get_media_url_str($category->name);
            }
            else if ($category->type == 'plugins')
            {
                $item_media_url = DuneStorePluginListScreen::get_media_url_str(
                    $category->name, true);
            }
            else
            {
                $item_media_url = DuneStoreSkinListScreen::get_media_url_str(
                    $category->name);
            }

            $items[] = array
            (
                PluginRegularFolderItem::media_url => $item_media_url,
                PluginRegularFolderItem::caption => $category->caption,
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $icon_path,
                    ViewItemParams::item_badge_icon_path => $category->badge_icon_url,
                    ViewItemParams::item_detailed_info => $item_detailed_info,
                    ViewItemParams::icon_margin_left => $category->icon_margin_left,
                    ViewItemParams::icon_margin_top => $category->icon_margin_top,
                    ViewItemParams::icon_margin_right => $category->icon_margin_right,
                    ViewItemParams::icon_margin_bottom => $category->icon_margin_bottom,
                    ViewItemParams::icon_sel_margin_left => $category->icon_margin_left,
                    ViewItemParams::icon_sel_margin_top => $category->icon_margin_top,
                    ViewItemParams::icon_sel_margin_right => $category->icon_margin_right,
                    ViewItemParams::icon_sel_margin_bottom => $category->icon_margin_bottom,
                ),
            );

            $ndx = $ndx + 1;
            if ($ndx > 9)
                $ndx = 1;
        }

        return $items;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
