<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';
require_once 'lib/dune_exception.php';
require_once 'lib/properties.php';

require_once 'dunestore_model.php';

class DuneStorePluginListScreen extends AbstractPreloadedRegularScreen
{
    const ID = 'plugin_list';

    public static function get_media_url_str($name, $show_installed)
    {
        return MediaURL::encode(
            array('screen_id' => self::ID,
                'name' => $name,
                'show_installed' => $show_installed));
    }

    ////////////////////////////////////////////////////////////

    private $model;

    public function __construct(DuneStoreModel $model)
    {
        parent::__construct(self::ID);
        $this->model = $model;
    }

    protected function do_get_folder_views(&$plugin_cookies)
    {
        $service_file_params = array(
            ViewItemParams::icon_path =>
            'plugin_file://icons/transparent.aai',
            ViewItemParams::icon_margin_left => 51,
            ViewItemParams::icon_margin_top => 51,
            ViewItemParams::icon_margin_right => 51,
            ViewItemParams::icon_margin_bottom => 0,
            ViewItemParams::icon_sel_margin_left => 51,
            ViewItemParams::icon_sel_margin_top => 51,
            ViewItemParams::icon_sel_margin_right => 51,
            ViewItemParams::icon_sel_margin_bottom => 0,
            ViewItemParams::item_badge_icon_path => "gui_skin://large_icons/4.png",
        );

        return array(
            // Index 0.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 2,

                    ViewParams::icon_selection_box_dx => -123,
                    ViewParams::icon_selection_box_dy => -131,
                    ViewParams::icon_selection_box_width => 245,
                    ViewParams::icon_selection_box_height => 245,

                    ViewParams::paint_icon_badge_box => 1,

                    ViewParams::icon_badge_box_dx => -100,
                    ViewParams::icon_badge_box_dy => -110,
                    ViewParams::icon_badge_box_width => 200,
                    ViewParams::icon_badge_box_height => 200,

                    ViewParams::icon_badge_box_sel_dx => -120,
                    ViewParams::icon_badge_box_sel_dy => -121,
                    ViewParams::icon_badge_box_sel_width => 240,
                    ViewParams::icon_badge_box_sel_height => 240,

                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_padding_top => 6,
                    ViewItemParams::item_padding_bottom => 28,
                    ViewItemParams::item_caption_dy => -10,
                    ViewItemParams::item_caption_sel_dy => 12,

                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    $service_file_params,
            ),

            // Index 2.
            // Index 1.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 3,

                    ViewParams::content_box_padding_top => -10,

                    ViewParams::icon_selection_box_dx => -98,
                    ViewParams::icon_selection_box_dy => -98,
                    ViewParams::icon_selection_box_width => 196,
                    ViewParams::icon_selection_box_height => 196,

                    ViewParams::paint_icon_badge_box => 1,

                    ViewParams::icon_badge_box_dx => -80,
                    ViewParams::icon_badge_box_dy => -80,
                    ViewParams::icon_badge_box_width => 160,
                    ViewParams::icon_badge_box_height => 160,

                    ViewParams::icon_badge_box_sel_dx => -96,
                    ViewParams::icon_badge_box_sel_dy => -96,
                    ViewParams::icon_badge_box_sel_width => 192,
                    ViewParams::icon_badge_box_sel_height => 192,

                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                    ViewItemParams::item_padding_top => 16,
                    ViewItemParams::item_padding_bottom => -15,
                    ViewItemParams::item_caption_dy => -20,
                    ViewItemParams::item_caption_sel_dy => 0,

                    ViewItemParams::icon_scale_factor => 0.8,
                    ViewItemParams::icon_sel_scale_factor => 0.96,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    $service_file_params,
            ),

            // Index 2.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 5,
                    ViewParams::num_rows => 3,

                    ViewParams::content_box_padding_top => -10,

                    ViewParams::icon_selection_box_dx => -98,
                    ViewParams::icon_selection_box_dy => -98,
                    ViewParams::icon_selection_box_width => 196,
                    ViewParams::icon_selection_box_height => 196,

                    ViewParams::paint_icon_badge_box => 1,

                    ViewParams::icon_badge_box_dx => -80,
                    ViewParams::icon_badge_box_dy => -80,
                    ViewParams::icon_badge_box_width => 160,
                    ViewParams::icon_badge_box_height => 160,

                    ViewParams::icon_badge_box_sel_dx => -96,
                    ViewParams::icon_badge_box_sel_dy => -96,
                    ViewParams::icon_badge_box_sel_width => 192,
                    ViewParams::icon_badge_box_sel_height => 192,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                    ViewItemParams::item_padding_top => 16,
                    ViewItemParams::item_padding_bottom => -15,
                    ViewItemParams::item_caption_dy => -20,
                    ViewItemParams::item_caption_sel_dy => 0,

                    ViewItemParams::icon_scale_factor => 0.8,
                    ViewItemParams::icon_sel_scale_factor => 0.96,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    $service_file_params,
            ),

            // Index 3.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 1,
                    ViewParams::num_rows => 11,
                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_caption_width => 1105,

                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_layout => HALIGN_LEFT,

                    ViewItemParams::icon_dx => 12,
                    ViewItemParams::icon_dy => -7,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    $service_file_params,
            ),
        );
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array
        (
            GUI_EVENT_KEY_ENTER => ActionFactory::open_folder(),
        );
    }

    private function get_effective_ui_type(&$settings)
    {
        if (isset($settings->ui_type))
            return $settings->ui_type;

        $skin_dir_props = new Properties("/tmp/dune_skin_dir.txt");
        $skin_dir_props->set_default('skin_dir', '/firmware/skin');
        $skin_dir = $skin_dir_props->skin_dir;
        $config_path = "/" . $skin_dir_props->skin_dir .
            "/dune_skin_config.xml";
        if (!is_file($config_path))
            $config_path = "/firmware/skin/dune_skin_config.xml";

        $doc = file_get_contents($config_path);
        if ($doc === false)
        {
            hd_print("Failed to read skin config file");
            return "type2";
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Failed to parse skin config file");
            return "type2";
        }

        if (!isset($xml->ui_types))
            return "type2";

        if (isset($xml->ui_types->default))
            return $xml->ui_types->default;

        if (isset($xml->ui_types->ui_type))
            return $xml->ui_types->ui_type;

        return "type2";
    }

    private function get_expand_main_screen_folders(&$settings)
    {
        $res = isset($settings->expand_main_screen_folders) ?
            ($settings->expand_main_screen_folders == 'yes') : true;
        hd_print("Expand_main_screen_items: ".($res ? 1 : 0));
        return $res;
    }

    private function load_menu_config($category)
    {
        $doc = file_get_contents(DuneStoreModel::MENU_CONFIG_FILE);
        if ($doc === false)
        {
            hd_print("Failed to read menu config file");
            return false;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Failed to parse menu config file");
            return false;
        }

        $parent_section_url = "root://".$category->name;
        $section_urls = array();

        foreach ($xml->sections->children() as $section)
        {
            $section_url = strval($section->url);
            $master_section_url = null;
            if (isset($section->master_section_url))
                $master_section_url = $section->master_section_url;

            if ($section_url == $parent_section_url ||
                $master_section_url == $parent_section_url)
            {
                $section_urls[] = $section_url;
            }
        }

        $categories = array();
        foreach ($section_urls as $section_url)
        {
            $catname = substr($section_url, 7);
            $category = $this->model->
                get_available_category_by_name($catname);
            if (isset($category))
                $categories[] = $category;
        }

        $show_slave_sections =
            isset($xml->show_slave_sections) &&
            strval($xml->show_slave_sections) == 'yes';

        hd_print("Num categories: ".count($categories));
        hd_print("Show_slave_sections: ".($show_slave_sections ? 1 : 0));

        return (object) array(
            'show_slave_sections' => $show_slave_sections,
            'categories' => $categories);
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        if (!$media_url->show_installed)
        {
            // Entering to "Add..." item.
            $this->model->unset_model();
        }

        $this->model->ensure_loaded();

        $cur_category = $this->model->get_available_category_by_name(
            $media_url->name);
        if (!isset($cur_category))
            return null;

        $view_index = $this->get_folder_view_index($plugin_cookies);
        $is_list_view = $view_index == 3;
        $use_description_in_details = false;

        $category_list = array($cur_category);
        if (!$media_url->show_installed)
        {
            $settings = new Properties("/config/settings.properties");

            $ui_type = $this->get_effective_ui_type($settings);
            hd_print("Effective ui_type: $ui_type");

            $expand_main_screen_folders =
                $this->get_expand_main_screen_folders($settings);

            $menu_config = $this->load_menu_config($cur_category);
            if ($menu_config !== false)
            {
                $show_slave_sections = $menu_config->show_slave_sections;
                if ($ui_type == 'type2' ||
                    !$show_slave_sections ||
                    !$expand_main_screen_folders)
                {
                    $category_list = $menu_config->categories;
                }
            }
        }

        $items = array();

        foreach ($category_list as $category)
        {
            foreach ($category->plugins as $plugin_name)
            {
                $plugin = $this->model->get_available_plugin_by_name($plugin_name);
                if (!isset($plugin))
                    continue;

                if (!$this->model->plugin_is_visible($plugin))
                {
                    continue;
                }

                $installed = $this->model->is_plugin_installed($plugin->name);
#            hd_print('name = ' . $plugin->name);
#            hd_print('caption = ' . $plugin->caption);
#            hd_print('icon_url = ' . $plugin->icon_url);
#            hd_print('installed = ' . $installed);

                if ($installed && !$media_url->show_installed)
                    continue;

                $installed_str = $installed ?
                    "%ext%<key_global>yes_no_choice_yes</key_global>" :
                    "%ext%<key_global>yes_no_choice_no</key_global>";

                if ($use_description_in_details)
                {
                    $item_detailed_info = $plugin->long_description;
                }
                else
                {
                    $item_detailed_info =
                        T::t('label_application') . '|' . $plugin->caption;
                    if (preg_match('/\S/', $plugin->short_description))
                    {
                        $item_detailed_info .=
                            ',|' . $plugin->short_description;
                    }
                    $item_detailed_info .=
                        '||' . T::t('label_installed') . '|' . $installed_str;
                    if ($installed)
                    {
                        $installed_plugin = $this->model->get_installed_plugin($plugin->name);
                        $installed_version = strval($installed_plugin->version);
                        if (preg_match('/\S/', $installed_version))
                        {
                            $item_detailed_info .=
                                '||' . T::t('label_installed_version') .
                                '|' . $installed_version;
                            
                        }
                    }
                    if (preg_match('/\S/', $plugin->version))
                    {
                        $item_detailed_info .=
                            '||' . T::t('label_version') . '|' . $plugin->version;
                    }
                }

                $starred = $installed && !$is_list_view;
                $icon_path = $is_list_view ?
                    'gui_skin://small_icons/service_file.aai' :
                    $plugin->icon_url;
                $item_detailed_icon_path = $is_list_view ?
                    $plugin->icon_url : null;

                $override_default_badge =
                    $this->model->is_plugin_badge_overridden($plugin->name);

                $items[] = array
                (
                    PluginRegularFolderItem::media_url =>
                        DuneStorePluginScreen::get_media_url_str(
                            $category->name, $plugin->name),
                    PluginRegularFolderItem::caption => $plugin->caption,
                    PluginRegularFolderItem::view_item_params => array
                    (
                        ViewItemParams::icon_path => $icon_path,
                        ViewItemParams::item_detailed_icon_path => $item_detailed_icon_path,
                        ViewItemParams::item_detailed_info => $item_detailed_info,
                        ViewItemParams::item_badge_icon_path => $plugin->badge_icon_url,
                        ViewItemParams::item_override_default_badge => $override_default_badge,
                    ),
                    PluginRegularFolderItem::starred => $starred,
                );
            }
        }

        return $items;
    }

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        $reg_view = parent::get_folder_view($media_url, $plugin_cookies);
        if ($media_url->show_installed)
            return $reg_view;

        $total = $reg_view[PluginFolderView::data]
            [PluginRegularFolderView::initial_range]
            [PluginRegularFolderRange::total];
        if ($total > 0)
            return $reg_view;

        $defs = array();
        ControlFactory::add_label($defs, null, T::key_global('no_more_apps_to_add'));
        ControlFactory::add_vgap($defs, 20);
        ControlFactory::add_custom_button($defs,
            ActionFactory::close_and_run(),
            "back", null, T::key_global('navigator_button_back'), 300);

        $folder_view = array
        (
            PluginControlsFolderView::defs => $defs,
            PluginControlsFolderView::initial_sel_ndx => -1,
        );

        return array
        (
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::archive                   => null,
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_CONTROLS,
            PluginFolderView::data                      => $folder_view,
        );
    }
}

///////////////////////////////////////////////////////////////////////////
?>
