<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';
require_once 'lib/dune_exception.php';

require_once 'dunestore_model.php';

class DuneStoreSkinListScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'skin_list';

    public static function get_media_url_str($name)
    {
        return MediaURL::encode(
            array('screen_id' => self::ID, 'name' => $name));
    }

    ////////////////////////////////////////////////////////////

    private $model;

    public function __construct(DuneStoreModel $model)
    {
        parent::__construct(self::ID);
        $this->model = $model;

        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    public function get_handler_id()
    { return self::ID; }

    ////////////////////////////////////////////////////////////

    protected function do_get_folder_views(&$plugin_cookies)
    {
        $service_file_params = array(
            ViewItemParams::icon_path =>
            'plugin_file://icons/transparent.aai',
            ViewItemParams::icon_margin_left => 51,
            ViewItemParams::icon_margin_top => 51,
            ViewItemParams::icon_margin_right => 51,
            ViewItemParams::icon_margin_bottom => 0,
            ViewItemParams::icon_sel_margin_left => 51,
            ViewItemParams::icon_sel_margin_top => 51,
            ViewItemParams::icon_sel_margin_right => 51,
            ViewItemParams::icon_sel_margin_bottom => 0,
        );

        return array(
            // Index 0.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 3,

                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 256,
                    ViewParams::sandwich_height => 144,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => false,

                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                    ViewItemParams::item_caption_dy => -15,
                    ViewItemParams::item_caption_sel_dy => -5,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array(
                    ViewItemParams::icon_path => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                ),
            ),

            // Index 1.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 2,

                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 384,
                    ViewParams::sandwich_height => 216,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => false,

                    ViewParams::paint_details => false,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                    ViewItemParams::item_caption_dy => -30,
                    ViewItemParams::item_caption_sel_dy => -15,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array(
                    ViewItemParams::icon_path => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                ),
            ),

            // Index 2.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 4,
                    ViewParams::num_rows => 4,

                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 240,
                    ViewParams::sandwich_height => 135,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => false,

                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array(
                    ViewItemParams::icon_path => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewItemParams::item_paint_caption => false,
                ),
            ),

            // Index 3.
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 1,
                    ViewParams::num_rows => 11,
                    ViewParams::paint_details => true,
                    ViewParams::paint_item_info_in_details => true,
                    ViewParams::item_detailed_info_auto_line_break => true,
                    ViewParams::item_detailed_info_rel_y => 200,
                    ViewParams::detailed_icon_valign => VALIGN_CENTER,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_caption_width => 1105,

                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_layout => HALIGN_LEFT,

                    ViewItemParams::icon_dx => 12,
                    ViewItemParams::icon_dy => -7,
                ),

                PluginRegularFolderView::not_loaded_view_item_params =>
                    $service_file_params,
            ),
        );
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array
        (
            GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action(
                $this, 'confirm_install'),
        );
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->model->ensure_loaded();

        $category = $this->model->get_available_category_by_name($media_url->name);
        if (!isset($category))
            return null;

        $view_index = $this->get_folder_view_index($plugin_cookies);
        $is_list_view = $view_index == 3;

        $items = array();

        foreach ($this->model->get_available_skins() as $skin)
        {
            if (!$this->model->skin_belongs_to_category($skin, $category))
                continue;

#            hd_print('name = ' . $skin->name);
#            hd_print('caption = ' . $skin->caption);
#            hd_print('icon_url = ' . $skin->icon_url);

            $item_detailed_info =
                T::t('label_skin') . '|' . $skin->caption;

            if (preg_match('/\S/', $skin->short_description))
                $item_detailed_info .= ',|' . $skin->short_description;

            $icon_path = $is_list_view ?
                'gui_skin://small_icons/appearance.aai' :
                $skin->icon_url;
            $item_detailed_icon_path = $is_list_view ?
                $skin->icon_url : null;

            $item_media_url = MediaURL::encode(
                array
                (
                    'screen_id' => 'skin',
                    'category' => $category->name,
                    'name'  => $skin->name,
                ));

            $items[] = array
            (
                PluginRegularFolderItem::media_url => $item_media_url,
                PluginRegularFolderItem::caption => $skin->caption,
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $icon_path,
                    ViewItemParams::item_detailed_icon_path => $item_detailed_icon_path,
                    ViewItemParams::item_detailed_info => $item_detailed_info,
                ),
            );
        }

        return $items;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('SkinListScreen: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $media_url = MediaURL::decode($user_input->selected_media_url);

        $skin = $this->model->get_available_skin_by_name(
            $media_url->name);
        if (!isset($skin))
            return null;

        if ($user_input->control_id == 'confirm_install')
        {
            $defs = array();
            ControlFactory::add_label($defs, null,
                T::key_global('gui_skin_install_dialog_will_be_installed'));
            ControlFactory::add_vgap($defs, 20);
            ControlFactory::add_close_dialog_and_apply_button($defs, $this, null,
                'install', T::key_global('button_ok'), 300,
                array('button_caption_centered' => true));
            ControlFactory::add_close_dialog_button($defs,
                T::key_global('button_cancel'), 300,
                array('button_caption_centered' => true));

            return ActionFactory::show_dialog(
                T::key_global('gui_skin_install_dialog_title'),
                $defs, true);
        }

        if ($user_input->control_id == 'install')
        {
            $url = 'skin_installer://' . $skin->download_url .
                ':::schema=zip&space_needed=' . $skin->space_needed;
            hd_print("Install URL = $url");
            return ActionFactory::launch_media_url($url);
        }

        return null;
    }

}

///////////////////////////////////////////////////////////////////////////
?>
